
/**********************************************************************************************************************/
/* This 3PC file will create an ASCII file with point type, chain name, station, offset, XY coordinates,              */
/*   and type of ROW for the ROW lines in plan view at a keyed in interval along the centerline chain.                */
/*   In addition, this file will compute row data at the endpoints of the row lines.  Lastly,                         */
/*   this file will compute row data at control points along the baseline.                                            */
/*   The ROW line is drawn in a plan view dgn file and is not a fixed distance                                        */
/*   from the centerline chain. Multiple instances of the same ROW type are allowed                                   */
/*   on the same side of the selected baseline.  This file can be used for metric and english plans.                  */
/*   This file will search for all 14 types of ROW specified in the define dgn statements.                            */
/*                                                                                                                    */
/*                                                                                                                    */
/* Basic Assumptions To Use This File                                                                                 */
/*    1. The user must hard code in the drawing name to use for searching for ROW                                     */
/*         lines in the define_dgn statements below on line number 37 only.                                           */
/*    2. The user must be inside this drawing with no reference files attached.                                       */
/*    3. The user must plot the station reference chain inside this drawing using D&C manager.                        */
/*    4. The user must run this criteria file from D&C manager where the "design keyin" dialog                        */
/*         states the following  "choose all;drop string;drop complex;choose all;criteria ascii_row.x; choose none"   */
/*         without the quotes.                                                                                        */
/*    5. --> IMPORTANT <-- This criteria requires that line strings and complex status in the file be dropped.        */
/*         Thus, a backup drawing should be utilized to run this criteria.                                            */
/*    6. This criteria file MUST be ran with Geopak 2000 or later.                                                    */
/*    7. If running Geopak 2000, the chain must begin before any right of way lines.  A syntax error will             */
/*         occur if this is not honored.  In Geopak 2001, the chain can begin after the right of way lines.           */
/*    8. --> IMPORTANT <-- See special comments on lines 70-81                                                        */
/*                                                                                                                    */
/**********************************************************************************************************************/

/********************** Symbology for point numbers in plan view *************************************************/
define "Increment Symbology"     lv=1 co=0 ft=5 wt=0 angle=45
define "End Point Symbology"     lv=1 co=1 ft=5 wt=0 angle=45
define "Control Point Symbology" lv=1 co=2 ft=5 wt=0 angle=45
/****************************************************************************************************************/   
   
/* The following line MUST be edited to show the drawing name to search */
define "RW Design File Name" row_report.dgn

/* These define statments MUST be revised for proper symbology per standards. DO NOT edit drawing name here      */
define_dgn "PROWRRTE" dgn = "RW Design File Name" lv=16 co=ByLevel wt=ByLevel /* Prop.  ROW - Rail Road Temporary Easement */
define_dgn "PROWRRPE" dgn = "RW Design File Name" lv=18 co=ByLevel wt=ByLevel /* Prop.  ROW - Rail Road Permanent Easement */
define_dgn "PROW"     dgn = "RW Design File Name" lvname=ROW Line co=ByLevel wt=ByLevel /* Prop.  ROW - Proposed Right of Way        */
define_dgn "PROWE"    dgn = "RW Design File Name" lvname=PERM Easement Line co=ByLevel wt=ByLevel /* Prop.  ROW - Permanent Easement           */
define_dgn "PROWTE"   dgn = "RW Design File Name" lvname=TEMP Easement Line co=ByLevel wt=ByLevel /* Prop.  ROW - Temporary Easement           */
define_dgn "PROWCA"   dgn = "RW Design File Name" lvname=Control of Access co=ByLevel wt=ByLevel /* Prop.  ROW - Controlled Access            */
define_dgn "PROWMR"   dgn = "RW Design File Name" lv=56 co=ByLevel wt=ByLevel /* Prop.  ROW - Minimum Right of Way         */
define_dgn "EROWRR"   dgn = "RW Design File Name" lv=14 co=ByLevel wt=ByLevel /* Exist. ROW - Rail Road                    */
define_dgn "EROWRRCL" dgn = "RW Design File Name" lv=15 co=ByLevel wt=ByLevel /* Exist. ROW - Rail Road Centerline         */
define_dgn "EROWRRE"  dgn = "RW Design File Name" lv=20 co=ByLevel wt=ByLevel /* Exist. ROW - Rail Road Easement           */
define_dgn "EROW"     dgn = "RW Design File Name" lvname=Ex:ROW co=ByLevel wt=ByLevel /* Exist. ROW - Existing Right of Way        */
define_dgn "EROWPL"   dgn = "RW Design File Name" lv=47 co=ByLevel wt=ByLevel /* Exist. ROW - Property Lines               */
define_dgn "EROWE"    dgn = "RW Design File Name" lvname=Ex:TEMP Easement Line co=ByLevel wt=ByLevel /* Exist. ROW - Easement Lines               */
define_dgn "EROWCA"   dgn = "RW Design File Name" lvname=Ex:Control of Access co=ByLevel wt=ByLevel /* Exist. ROW - Controlled Access            */


main
{
 /* Text size for point labels in plan view. */ 
 _d_text_size = 2 

 /* Default entries for prompts */
 _s_job_number = "590"
 _s_chain_name = "Mainline"
 _d_inc = 25
 _s_ascii_file = "row report.txt"
 _d_max_offset = 500 /* Filter for limiting offset */


 
 _d_limit = 300 /* Moves pointer to check for additional ROW lines beyond the 300 foot barrier */
 _d_iterations = 2 /* Number of iterations to move beyond the 300 foot plan search limitation 5 will yield 1800' offset*/
 
 /****************************************************************************************************************/
 /* NOTE ---> The above two variables work together.  There is a limitation in Geopak that when searching for    */
 /* a plan view element, it will  only loook for 300 feet.  The above two variables work around this limitation  */
 /* in the following manor.  "_d_limit" instructs Geopak that after you look 300 feet for the intersection of a  */
 /* plan view element, to skip out a distance of "_d_limit" and look another 300 feet.  The second variable      */
 /* instructs Geopak on how many times do you want to do this.  IMPORTANT ----> Make sure that in an alignement  */
 /* that loops back on itself, to not let the search and iterations go back and intersect a previous part of the */
 /* alignment.  If you are not using a "Hairpin" alignment this is not an issue.                                 */   
 /****************************************************************************************************************/
 

PROMPT ^Keyin Maximum Offset^ VARIABLE=_d_max_offset MODE=KEYIN
if (CANCEL) then {EXIT}

_ss_set = dgn selection set 
_ed_element = _ss_set(1)

while _ed_element  /* Throws out all text and cells */
  {
    _ed_tmp_element = _ss_set(_ed_element + 1)
   if ((_ed_element.type=text) or (_ed_element.type=cell)) then 
      {
         _ss_set = _ss_set - _ed_element
      }

     _ed_element = _ed_tmp_element
  }

_ed_element = _ss_set(1)

while _ed_element
  {
    /* Filters Total Selection Set To Only Include Right of Way Elements. --> Must match design standards from above <-- */
   _ed_tmp_element = _ss_set(_ed_element + 1)
   if not (((_ed_element.weight=3) and (_ed_element.level=52) and (_ed_element.color=117)) /* Prop. ROW */
       or  ((_ed_element.weight=1) and (_ed_element.level=48) and (_ed_element.color=54))  /* Exist. ROW - Existing Right of Way */
       or  ((_ed_element.weight=1) and (_ed_element.level=16) and (_ed_element.color=114)) /* Prop. ROW - Rail Road Temporary Easement */
       or  ((_ed_element.weight=1) and (_ed_element.level=18) and (_ed_element.color=6))   /* Prop. ROW - Rail Road Permanent Easement */
       or  ((_ed_element.weight=1) and (_ed_element.level=53) and (_ed_element.color=6))   /* Prop. ROW - Permanent Easement */
       or  ((_ed_element.weight=1) and (_ed_element.level=61) and (_ed_element.color=114)) /* Prop. ROW - Temporary Easement */
       or  ((_ed_element.weight=1) and (_ed_element.level=55) and (_ed_element.color=117)) /* Prop. ROW - Controlled Access */
       or  ((_ed_element.weight=3) and (_ed_element.level=56) and (_ed_element.color=3))   /* Prop. ROW - Minimum Right of Way */
       or  ((_ed_element.weight=1) and (_ed_element.level=14) and (_ed_element.color=6))   /* Exist. ROW - Rail Road */
       or  ((_ed_element.weight=1) and (_ed_element.level=15) and (_ed_element.color=6))   /* Exist. ROW - Rail Road Centerline */
       or  ((_ed_element.weight=1) and (_ed_element.level=20) and (_ed_element.color=1))   /* Exist. ROW - Rail Road Easement */
       or  ((_ed_element.weight=5) and (_ed_element.level=47) and (_ed_element.color=73))  /* Exist. ROW - Property Lines */
       or  ((_ed_element.weight=1) and (_ed_element.level=49) and (_ed_element.color=6))   /* Exist. ROW - Easement Lines */
       or  ((_ed_element.weight=1) and (_ed_element.level=42) and (_ed_element.color=54))  /* Exist. ROW - Controlled Access */
       or  ((_ed_element.weight=3) and (_ed_element.level=1 ) and (_ed_element.color=2))) then /* Proposed Station Reference Baseline */
      {
         _ss_set = _ss_set - _ed_element
      }

     _ed_element = _ed_tmp_element
  }


_d_unknown = 0
_d_point_number = 0
_d_max = 1 /* Counter to check number of iterations */
_s_point_type = "n/a"

/* must define Job Number */
PROMPT ^Keyin GPK Job Number^ VARIABLE=_s_job_number MODE=KEYIN
if (CANCEL) then {EXIT}

SET JOB NUMBER _s_job_number

PROMPT ^Keyin Chain Name^ VARIABLE=_s_chain_name MODE=KEYIN
if (CANCEL) then {EXIT}

_s_label = "Keyin Desired Extraction Increment"
PROMPT _s_label VARIABLE=_d_inc MODE=KEYIN
if (CANCEL) then {EXIT}

_s_label = "Keyin ASCII File Name"
PROMPT _s_label VARIABLE=_s_ascii_file MODE=KEYIN
if (CANCEL) then {EXIT}

_s_ascii_file = _s_chain_name + "_" + _s_ascii_file

ASCII open file = _s_ascii_file

/* Find the end of the chain */
_s_end_sta = CVT_distAlongChain_Sta (1000000, _s_chain_name)
_s_end_reg = CVT_distAlongChain_Reg (1000000, _s_chain_name)
_d_end_dist_along = CVT_StaReg_distAlongChain (_s_end_sta, _s_end_reg, _s_chain_name)
_d_total_distance = _d_end_dist_along

/* Set starting station to provide even station values */
_s_beg_sta = CVT_distAlongChain_Sta (0, _s_chain_name)
prompt ^Keyin Starting Station Value^ variable = _s_beg_sta mode = keyin
if (CANCEL) then {EXIT}
_s_beg_reg = CVT_distAlongChain_Reg (0, _s_chain_name)
prompt ^Select Starting Station Region Value^ variable = _s_beg_reg mode = option content = 1,2,3,4,5,6,7,8,9,10
if (CANCEL) then {EXIT}

/* Set Ending Station if it does not match the end of the chain */
prompt ^Keyin Ending Station Value^ variable = _s_end_sta mode = keyin
if (CANCEL) then {EXIT}
prompt ^Select Ending Station Region Value^ variable = _s_end_reg mode = option content = 1,2,3,4,5,6,7,8,9,10
if (CANCEL) then {EXIT}
_d_end_dist_along = CVT_StaReg_distAlongChain (_s_end_sta, _s_end_reg, _s_chain_name)

_d_beg_dist_along = CVT_StaReg_distAlongChain (_s_beg_sta, _s_beg_reg, _s_chain_name)
_d_cur_dist_along = _d_beg_dist_along

call subroutine points            /* Part 1 increment points        */
call subroutine end_points        /* Part 2 ROW end points          */       
call subroutine control_points    /* Part 3 Baseline Control Points */
draw skip to chain _s_chain_name distalongchain=0 off=0
draw skip to chain _s_chain_name distAlongChain=_d_end_dist_along OFF=0

if ((_d_total_distance - _d_end_dist_along) < 2* tolerance) then
 {
   call subroutine points2  /* Takes care of ending point of chain */
 }  

ASCII CLOSE FILE = _s_ascii_file

Prompt ^Process Completed Successfully!^  mode = message

_s_temp = ^filename.txt^
ASCII OPEN READONLY FILE = _s_temp
ASCII READ FILE = _s_temp
format = ^%s^
var = _s_filename
ASCII CLOSE FILE = _s_temp

_s_command = ^rd=^ + _s_filename
SEND MICROSTATION _s_command
}

sub control_points
{
 /* _ss_set = dgn selection set */
  _ed_element = _ss_set(1)
    while _ed_element
     {
      if (_ed_element.level=1) then
        {
          _s_point_type = "Control Points"
          mark _mp_beg
          _mp_beg.x = _ed_element.x1
          _mp_beg.y = _ed_element.y1
          _d_beg_dac = cogo_distalongchain(_mp_beg,_s_chain_name)
          mark _mp_end
          _mp_end.x = _ed_element.x2
          _mp_end.y = _ed_element.y2
          _d_end_dac = cogo_distalongchain(_mp_end,_s_chain_name)
          call sub point_switch
          draw skip to _mp_beg
          mark _mp_cur
          _d_cur_dist_along = cogo_distalongchain(_mp_beg,_s_chain_name)
       
            if (_d_cur_dist_along > _d_end_dist_along) then
              {
               _ed_element = _ss_set(_ed_element + 1) 
              }
            else
              {
                call sub points2
                _ed_element = _ss_set(_ed_element + 1)
              }
        } 
      else
        {
          _ed_element = _ss_set(_ed_element + 1)
        }
     
     }
}
end sub


sub points2
{
 if (1=1) then
  {
   _s_point_type = "control"
   mark _mp_cur_dist_along
   mark _mp_temp
   _d_tangent_angle = cogo_tangentAngleChain (_mp_cur_dist_along,_s_chain_name)
   _d_turn_angle = _d_tangent_angle+90
   _d_counter = 1
   _d_move = 5*tolerance
   
   while _d_counter <= 2
   { 
     _d_flag = 0
     if (_d_counter = 1) then
       {
         _s_side = "left"
       }
     else
       {
         _s_side = "right"
       }    
     while _d_flag = 0
       {
         draw skip angle = _d_turn_angle to "PROW"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > .1) then
          {
            _s_row_type = "New ROW"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
          } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
        }
      
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROW"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing ROW"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWRRTE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New RR Temp Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWRRPE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New RR Perm Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }     
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New Perm Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }     
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWTE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New Temp Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }     
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWCA"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New C.A."
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }     
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWMR"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New Minimum ROW Line"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWRR"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing RR ROW Line"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWRRCL"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing RR CL"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWRRE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing RR Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWPL"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing Prop Line"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWCA"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing C.A."
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_max = 1
            draw skip to _mp_cur_dist_along
            mark _mp_temp
            _d_flag = 1
           }
       }
     
      _d_counter = _d_counter + 1
      _d_turn_angle = _d_turn_angle - 180  
     /* _d_move=_d_move*-1
      _d_limit=_d_limit*-1 */
   }
  
  }
}
end sub



sub points
{

while ((_d_cur_dist_along + 1.0) < _d_end_dist_along)
 {
   _s_point_type = "incremt"
   draw skip to chain _s_chain_name distAlongChain=_d_cur_dist_along OFF=0
   mark _mp_cur_dist_along
   mark _mp_temp
   _d_tangent_angle = cogo_tangentAngleChain (_mp_cur_dist_along,_s_chain_name)
   _d_turn_angle = _d_tangent_angle+90
   _d_counter = 1
   _d_move = 5*tolerance
   
   while _d_counter <= 2
   { 
     _d_flag = 0
     if (_d_counter = 1) then
       {
         _s_side = "left"
       }
     else
       {
         _s_side = "right"
       }    
     while _d_flag = 0
       {
         draw skip angle = _d_turn_angle to "PROW"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > .1) then
          {
            _s_row_type = "New ROW"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
          } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
        }
      
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROW"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing ROW"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWRRTE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New RR Temp Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWRRPE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New RR Perm Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }     
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New Perm Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }     
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWTE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New Temp Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }     
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWCA"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New C.A."
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }     
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "PROWMR"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "New Minimum ROW Line"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWRR"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing RR ROW Line"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWRRCL"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing RR CL"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWRRE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing RR Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWPL"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing Prop Line"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWE"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing Ease"
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
          }
       }
     
     draw skip to _mp_cur_dist_along
     mark _mp_temp
     _d_flag = 0
     while _d_flag = 0
       {     
         draw skip angle = _d_turn_angle to "EROWCA"
         mark _mp_cur
         _d_dist_between = cogo_distance_between_points (_mp_temp, _mp_cur)
         if (_d_dist_between > 0.1) then
           {
            _s_row_type = "Existing C.A."
            call sub write_ascii
            draw skip angle=_d_turn_angle for distance =_d_move
            mark _mp_temp
           } 
         else if (_d_max <= _d_iterations) then
          {
           _d_max=_d_max+1
           draw skip angle=_d_turn_angle for distance =_d_limit
           mark _mp_temp
          }
         else
          {
            _d_flag = 1        
            _d_max = 1
            draw skip to _mp_cur_dist_along
            mark _mp_temp            
          }
       }
     
      _d_counter = _d_counter + 1
      _d_turn_angle = _d_turn_angle - 180  
      /* _d_move=_d_move*-1
      _d_limit=_d_limit*-1 */
  
   }
  
   _d_cur_dist_along = _d_cur_dist_along + _d_inc
 
 }
}
end sub


sub write_ascii
 {            
   if not _s_row_type = "UNKNOWN" then   
     {
       if not _s_point_type = "endpont" then
         {
           _d_cur_dist_along = cogo_distalongchain(_mp_cur_dist_along, _s_chain_name)
           _d_off = abs(cogo_distance_between_points (_mp_cur, _mp_cur_dist_along))
           if (_s_side = "left") then
             {
               _d_off=_d_off * -1
             }
         }
       else
         {
           _d_off = cogo_offsetFromChain (_mp_cur, _s_chain_name)
         }
       _d_region = cvt_distAlongChain_Reg (_d_cur_dist_along,_s_chain_name)
       _s_region = CVT_D_S0(_d_region)
       _s_cur_sta = CVT_distAlongChain_Sta (_d_cur_dist_along, _s_chain_name) + " R" + _s_region
       _d_x = _mp_cur.x
       _d_y = _mp_cur.y
         if (_d_cur_dist_along <= _d_end_dist_along) and (abs(_d_off) <= _d_max_offset) and
            (_d_cur_dist_along >= _d_beg_dist_along) then
           {   
             _d_point_number = _d_point_number + 1
             ASCII WRITE FILE = _s_ascii_file
             FORMAT = ^%10.0f,%s, %s, %s,%10.2f,%15.4f,%15.4f, %s\n^
             VAR = _d_point_number,_s_point_type,_s_chain_name,_s_cur_sta,_d_off,_d_x,_d_y,_s_row_type
             _s_point_number = cvt_d_s0(_d_point_number)
             if (_s_point_type = "endpont") then
               {
                 draw gg=^text^ text value = _s_point_number "End Point Symbology" th=_d_text_size tw=_d_text_size just=cc  
               }
             else if (_s_point_type = "incremt") then
               {
                 draw gg=^text^ text value = _s_point_number "Increment Symbology" th=_d_text_size tw=_d_text_size just=cc    
               }
             else if (_s_point_type = "control") then
               {
                 draw gg=^text^ text value = _s_point_number "Control Point Symbology" th=_d_text_size tw=_d_text_size just=cc                   
               } 
           }
         else
          {
           draw skip dx =0 dy=0
          }    
     }
      
 }
end sub  
    
sub end_points
{
/* _ss_set = dgn selection set */
  _ed_element = _ss_set(1)
  _d_count1 = 1
   
  /* Loop over element approach */
  while (_ed_element) 
    {
      if not (_ed_element.level = 1) then /* Excludes Baseline On Level 42 */
             
        {
          _s_point_type = "endpont"
          mark _mp_beg
          _mp_beg.x = _ed_element.x1
          _mp_beg.y = _ed_element.y1
          _d_beg_dac = cogo_distalongchain(_mp_beg,_s_chain_name)
          mark _mp_end
          _mp_end.x = _ed_element.x2
          _mp_end.y = _ed_element.y2
          _d_end_dac = cogo_distalongchain(_mp_end,_s_chain_name)
          call sub row_type
          call sub point_switch
          draw skip to _mp_beg
          mark _mp_cur
          _d_cur_dist_along = _d_beg_dac
          call sub write_ascii
          draw skip to _mp_end
          mark _mp_cur
          _d_cur_dist_along = _d_end_dac
         
          _ed_element2 = _ss_set(_d_count1+1)
          _d_stop = 0
          
           while (_ed_element2) and (_d_stop = 0)
              {
                mark _mp_beg2
                _mp_beg2.x = _ed_element2.x1
                _mp_beg2.y = _ed_element2.y1
                _d_beg2_dac = cogo_distalongchain(_mp_beg2,_s_chain_name)
                mark _mp_end2
                _mp_end2.x = _ed_element2.x2
                _mp_end2.y = _ed_element2.y2
                _d_end2_dac = cogo_distalongchain(_mp_end2,_s_chain_name)
                call sub point_switch2     
                  if (abs(_mp_beg2.x-_mp_end.x) < 5*tolerance and
                     abs(_mp_beg2.y-_mp_end.y) < 5*tolerance) then 
                    {
                      _d_stop = 1
                    } 
                                      
                   else
                    {
                      _ed_element2 = _ss_set(_ed_element2 + 1)
                    }
              } 
            
            _ed_element2 = _ss_set(1)
            _d_count2 = 1
            
            while (_d_count2 < _d_count1) and (_d_stop = 0)
              {
                mark _mp_beg2
                _mp_beg2.x = _ed_element2.x1
                _mp_beg2.y = _ed_element2.y1
                _d_beg2_dac = cogo_distalongchain(_mp_beg2,_s_chain_name)
                mark _mp_end2
                _mp_end2.x = _ed_element2.x2
                _mp_end2.y = _ed_element2.y2
                _d_end2_dac = cogo_distalongchain(_mp_end2,_s_chain_name)
                call sub point_switch2     
                  if (abs(_mp_beg2.x-_mp_end.x) < 5*tolerance and
                     abs(_mp_beg2.y-_mp_end.y) < 5*tolerance) then 
                    {
                      _d_stop = 1
                    } 
                                      
                   else
                    {
                      _ed_element2 = _ss_set(_ed_element2 + 1)
                    }
                 _d_count2 = _d_count2 + 1
              } 
            
            
            if (_d_stop = 0) then
              {
                draw skip to _mp_end
                mark _mp_cur
                _d_cur_dist_along = _d_end_dac
                call sub write_ascii
              }
          
       }   
           
     _ed_element = _ss_set(_ed_element + 1)
     _d_count1 = _d_count1 + 1      
   }       
           
}          
end sub    

       
sub row_type
{
  if ((_ed_element.level = 16) and (_ed_element.color = 114) and (_ed_element.weight = 1)) then
    {
      _s_row_type = "New RR Temp Ease"
    }
  else if ((_ed_element.level = 18) and (_ed_element.color = 6)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "New RR Perm Ease"
    }
  else if ((_ed_element.level = 52) and (_ed_element.color = 117)  and (_ed_element.weight = 3)) then
    {
      _s_row_type = "New ROW"
    }
  else if ((_ed_element.level = 53) and (_ed_element.color = 6)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "New Perm Ease"
    }
  else if ((_ed_element.level = 61) and (_ed_element.color = 114)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "New Temp Ease"
    }
  else if ((_ed_element.level = 55) and (_ed_element.color = 117)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "New C.A."
    }
  else if ((_ed_element.level = 56) and (_ed_element.color = 3)  and (_ed_element.weight = 3)) then
    {
      _s_row_type = "New Minimum ROW Line"
    }
  else if ((_ed_element.level = 14) and (_ed_element.color = 6)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "Existing RR ROW Line"
    }
  else if ((_ed_element.level = 15) and (_ed_element.color = 6)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "Existing RR CL"
    }
  else if ((_ed_element.level = 20) and (_ed_element.color = 1)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "Existing RR Ease"
    }
  else if ((_ed_element.level = 48) and (_ed_element.color = 54)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "Existing ROW"
    }
  else if ((_ed_element.level = 47) and (_ed_element.color = 73)  and (_ed_element.weight = 5)) then
    {
      _s_row_type = "Existing Prop Line"
    }
  else if ((_ed_element.level = 49) and (_ed_element.color = 6)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "Existing Ease"
    }
  else if ((_ed_element.level = 42) and (_ed_element.color = 54)  and (_ed_element.weight = 1)) then
    {
      _s_row_type = "Existing C.A."
    }
  else
    {
      _s_row_type = "UNKNOWN"
    }
  
}
end sub

sub point_switch
{
if (_d_end_dac < _d_beg_dac ) then  /* Backwards stationing reverse points */   
  {                                                                             
    _mp_beg.x = _ed_element.x2                                                  
    _mp_beg.y = _ed_element.y2                                                  
    _d_beg_dac = cogo_DistAlongChain(_mp_beg,_s_chain_name)                       
    _mp_end.x = _ed_element.x1                                                  
    _mp_end.y = _ed_element.y1                                                  
    _d_end_dac = cogo_distalongchain(_mp_end,_s_chain_name)                     
  }
  
}
end sub

sub point_switch2                                                                
{                                                                               
if (_d_end2_dac < _d_beg2_dac ) then  /* Backwards stationing reverse points */                                                                                  
  {                                                                             
    _mp_beg2.x = _ed_element2.x2                                                  
    _mp_beg2.y = _ed_element2.y2                                                  
    _d_beg2_dac = cogo_DistAlongChain(_mp_beg2,_s_chain_name)                     
    _mp_end2.x = _ed_element2.x1                                                  
    _mp_end2.y = _ed_element2.y1                                                  
    _d_end2_dac = cogo_distalongchain(_mp_end,_s_chain_name)                     
  }                                                                             
                                                                                
}                                                                               
end sub                                                                         



